#include <system/ablib_system.h>
#include "ddeclient.h"

namespace ablib{
namespace system{

namespace{
	inline HSZ DdeCreateStrHandle(DWORD ddeinst,const TCHAR *str){
#ifdef UNICODE
		return ::DdeCreateStringHandle(ddeinst,str,CP_WINUNICODE);
#else
		return ::DdeCreateStringHandle(ddeinst,str,CP_WINANSI);
#endif
	}
}

DdeClient::DdeClient(void) : m_ddeinst(0),m_hconv(0){
	if(::DdeInitialize(&m_ddeinst,sDdeCallBack,APPCMD_CLIENTONLY, 0) != DMLERR_NO_ERROR)m_ddeinst = 0;
}

DdeClient::~DdeClient(void){
	Terminate();
	::DdeUninitialize(m_ddeinst);
	m_ddeinst = 0;
}

bool DdeClient::Init(const ablib::string &app,const ablib::string &topic){
	if(m_ddeinst == 0)return false;
	if(m_hconv)return false;
	m_service = DdeCreateStrHandle(m_ddeinst,app.c_str());
	m_topic = DdeCreateStrHandle(m_ddeinst,topic.c_str());
	m_hconv = ::DdeConnect(m_ddeinst,m_service,m_topic,NULL);
	if(m_hconv == 0L){
		::DdeFreeStringHandle(m_ddeinst,m_service);
		::DdeFreeStringHandle(m_ddeinst,m_topic);
		return false;
	}
	m_servicestr = app;
	m_topicstr = topic;
	return true;
}

bool DdeClient::Terminate(){
	if(m_hconv){
		::DdeFreeStringHandle(m_ddeinst,m_service);
		::DdeFreeStringHandle(m_ddeinst,m_topic);
		::DdeDisconnect(m_hconv);m_hconv = 0;
		return true;
	}else return false;
}

ablib::string DdeClient::Request(const ablib::string &item){
	HSZ hsz = DdeCreateStrHandle(m_ddeinst,item.c_str());
	HDDEDATA hdata = ::DdeClientTransaction(NULL,0,m_hconv,hsz,CF_TEXT,XTYP_REQUEST,9999,NULL);
	if(!hdata){
		::DdeFreeStringHandle(m_ddeinst,hsz);
		return _T("");
	}
	ablib::string rv = reinterpret_cast<TCHAR*>(::DdeAccessData(hdata,NULL));
	::DdeUnaccessData(hdata);
	::DdeFreeDataHandle(hdata);
	::DdeFreeStringHandle(m_ddeinst,hsz);
	return rv;
}

int DdeClient::Poke(const ablib::string &item,const ablib::string &param){
	HSZ hsz = DdeCreateStrHandle(m_ddeinst,item.c_str());
	int rv = reinterpret_cast<int>(::DdeClientTransaction(reinterpret_cast<BYTE*>(const_cast<TCHAR*>(param.c_str())),param.length() + 1,m_hconv,hsz,CF_TEXT,XTYP_POKE,9999,NULL));
	::DdeFreeStringHandle(m_ddeinst,hsz);
	return rv;
}

int DdeClient::Execute(const ablib::string &exec){
	return reinterpret_cast<int>(::DdeClientTransaction(reinterpret_cast<BYTE*>(const_cast<TCHAR*>(exec.c_str())),exec.length() + 1,m_hconv,0L,0,XTYP_EXECUTE,9999,NULL));
}

HDDEDATA CALLBACK DdeClient::sDdeCallBack(UINT uType,UINT uFmt,HCONV hconv,HSZ hsz1,HSZ hsz2,HDDEDATA hdata,DWORD dwData1,DWORD dwData2){
	return reinterpret_cast<HDDEDATA>(TRUE);
}


}//namespace system
}//namespace ablib

